{ Written by Alexander N. Naumov (2:5020/423.11@fidonet) }

{$N+,E-}
uses
  Int13X, Huge, Timer;

const
  BufLen = $8000;

var
  Drive: Byte;
  Param: XParameters;

procedure Verify;
var
  Size, TimerTicks, TotalTime, ReadSpeed, Head : LongInt;
  Time: Word;
  Count: Word;
  Code: Byte;
  Packet: xPacket;
  Block: HugeInt;
  TestBuffer: Pointer;
begin
  GetMem(TestBuffer, BufLen);

  TimerTicks := 0; TotalTime := 0; Size := 0;
  Block[0] := 0;
  Block[1] := 0;
  Count := BufLen div Param.BytesPerSector;

  repeat
    Count := HugeStep(Block, Param.Sectors, Count);

    if Count = 0 then Break;

    Packet.Count  := Count;
    Packet.Buffer := TestBuffer;
    Packet.Start  := Block;

    StartTimer;
    Code := RequestDrive(Drive, xVerify, Packet);
    Time := ReadTimer;

    if Code = 0 then
      Inc(TimerTicks, Time);

    Inc(Size, Packet.Count*Param.BytesPerSector);

    if TimerTicks > $100000 then
    begin
      Write(#13, HugeToReal(Block)*100.0/HugeToReal(Param.Sectors) :3:0, '% ', Size div (TimerTicks div 1193) :6, ' KBytes/s');
      Inc(TotalTime, TimerTicks div 1193);
      TimerTicks := 0; Size := 0;
    end;

    if (Code <> 0) or (Packet.Count <> Count) then
    begin
      WriteLn(#9'Error!');
      Break;
    end;

    HugeInc(Block, Count);

  until False;

  Inc(TotalTime, TimerTicks div 1193);
  Write(#13#10, 'Average Linear Read Speed:', HugeToReal(Param.Sectors)*Param.BytesPerSector/TotalTime :6:0, ' KBytes/s'#7);

end;

procedure Help;
begin
  WriteLn('xVerify [drive]'#13#10'drive: 0..9');
  Halt;
end;

procedure Error(const Msg: String; ErrCode: Word);
begin
  Write('ERROR! ');
  WriteLn(Msg);
  Halt(ErrCode);
end;

var
  Drv: String[2];
  Info: XConfig;
begin
  Drv := ParamStr(1);
  if (Length(Drv) <> 1) or not (Drv[1] in ['0'..'3']) then
    Help;

  Drive := Ord(Drv[1]) - Ord('0') + $80;

  if not CheckDrive(Drive, Info) then
    Error('INT 13 Extensions not supported.', 1);

  if Info.Support and xAccess = 0 then
    Error('Extended disk access functions not supported.', 2);

  if GetDriveParameters(Drive, Param) <> 0 then
    Error('Drive parameters unknown.', 3);

  Verify;
end.
