{ Written by Alexander N. Naumov (2:5020/423.11@fidonet)               }
{ Portions Copyright (C) 1996-99 Michael Radchenko (2:5025/25@fidonet) }

{$N+,E-,X+}
uses
  Int13X, Timer;

const
  BufLen = $8000;

const
  reqRead: Word = xRead;

var
  Drive: Byte;
  Param: XParameters;
  TestBuffer: Pointer;

function DriveSeek(Cylinder, Head, Sector: LongInt): Byte;
var Packet: XPacket;
begin
  Packet.Count  := 0;
  Packet.Buffer := nil;
  CHStoBlock(Param.Heads, Param.SectorsPerTrack, Cylinder, Head, Sector, Packet.Start);
  DriveSeek := RequestDrive(Drive, xSeek, Packet);
end;

function DriveRead(Cylinder, Head, Sector: LongInt;
                   Buffer: Pointer; Count: Word): Byte;
var
  Packet: XPacket;
begin
  Packet.Count  := Count;
  Packet.Buffer := Buffer;
  CHStoBlock(Param.Heads, Param.SectorsPerTrack, Cylinder, Head, Sector, Packet.Start);
  DriveRead := RequestDrive(Drive, reqRead, Packet);
end;

function GetAverageAccessTime: Double;
var
  I, TimerTicks: LongInt;
begin
  DriveSeek(Param.Cylinders div 2, 0, 1);

  TimerTicks := 0;
  for I := 1 to 1000 do begin
    StartTimer;
    DriveRead(Random(Param.Cylinders), Random(Param.Heads), Random(Param.SectorsPerTrack)+1,
              TestBuffer, 1);
    Inc(TimerTicks, ReadTimer);
  end;

  GetAverageAccessTime := TimerTicks/(1193*1000);
end;

function GetAverageSeekTime: Double;
var
  I, TimerTicks: LongInt;
begin
  DriveSeek(Param.Cylinders div 2, 0, 1);

  TimerTicks := 0;
  for I := 1 to 1000 do begin
    StartTimer;
    DriveSeek(Random(Param.Cylinders), Random(Param.Heads), Random(Param.SectorsPerTrack)+1);
    Inc(TimerTicks, ReadTimer);
  end;

  GetAverageSeekTime := TimerTicks/(1193*1000);
end;

function GetMaxSeekTime: Double;
var
  I, TimerTicks: LongInt;
begin
  TimerTicks := 0;

  for I := 1 to 300 do begin
    StartTimer;
    DriveSeek(0, 0, 1);
    Inc(TimerTicks, ReadTimer);

    StartTimer;
    DriveSeek(Param.Cylinders-1, 0, 1);
    Inc(TimerTicks, ReadTimer);
  end;

  GetMaxSeekTime := TimerTicks/(1193*600);
end;

function GetTrackToTrackSeekTime: Double;
var
  I, TimerTicks: LongInt;
begin
  DriveSeek(0, 0, 1);

  TimerTicks := 0;
  for I := 0 to Param.Cylinders-1 do begin
    StartTimer;
    DriveSeek(I, 0, 1);
    Inc(TimerTicks, ReadTimer);
  end;

  GetTrackToTrackSeekTime := TimerTicks/(1193*(Param.Cylinders-1));
end;

function GetTrackLinearReadSpeed(Track: LongInt): LongInt;
var
  Size, TimerTicks, Head: LongInt;
  Time: Word;
  Code: Byte;
begin
  if Track > 0 then
    DriveRead(Track-1, Param.Heads-1, Param.SectorsPerTrack, TestBuffer, 1);

  TimerTicks := 0; Size := 0;
  for Head := 0 to Param.Heads-1 do begin
    StartTimer;
    Code := DriveRead(Track, Head, 1, TestBuffer, Param.SectorsPerTrack);
    Time := ReadTimer;
    if Code = 0 then Inc(TimerTicks, Time);

    Inc(Size, Param.SectorsPerTrack*Param.BytesPerSector);
  end;

  GetTrackLinearReadSpeed := Size div (TimerTicks div 1193);
end;

procedure GetLinearReadSpeed(var AverageReadSpeed, MinReadSpeed, MaxReadSpeed: LongInt);
var
  I, N, TrackSpeed : LongInt;
begin
  AverageReadSpeed := 0;
  MinReadSpeed := High(Integer); MaxReadSpeed := 0;
  N := Param.Cylinders div 20;
  for I := 0 to N-1 do begin
    TrackSpeed := GetTrackLinearReadSpeed(I*20);
    if TrackSpeed > MaxReadSpeed then MaxReadSpeed := TrackSpeed;
    if TrackSpeed < MinReadSpeed then MinReadSpeed := TrackSpeed;
    AverageReadSpeed := AverageReadSpeed+TrackSpeed;
  end;
  AverageReadSpeed := AverageReadSpeed div N;
end;

procedure FakeDriveGeometry(var Param: XParameters);
begin
  Param.Cylinders := Trunc(Param.Sectors/(63*255));
  Param.Heads := 255;
  Param.SectorsPerTrack := 63;
end;

procedure Help;
begin
  WriteLn('xHDDSpd [drive]'#13#10'drive: 0..9');
  Halt;
end;

procedure Error(const Msg: String; ErrCode: Word);
begin
  Write('ERROR! ');
  WriteLn(Msg);
  Halt(ErrCode);
end;

procedure Warning(const Msg: String);
begin
  Write('WARNING! ');
  WriteLn(Msg);
end;

var
  Drv: String[2];
  Info: XConfig;
  B: HugeInt;
  AverageReadSpeed, MinReadSpeed, MaxReadSpeed: LongInt;
  AverageAccess: Double;
begin
  Drv := ParamStr(1);
  if (Length(Drv) <> 1) or not (Drv[1] in ['0'..'9']) then
    Help;

  Drive := Ord(Drv[1]) - Ord('0') + $80;

  if not CheckDrive(Drive, Info) then
    Error('INT 13 Extensions not supported.', 1);

  if Info.Support and xAccess = 0 then
    Error('Extended disk access functions not supported.', 2);

  if GetDriveParameters(Drive, Param) <> 0 then
    Error('Drive parameters unknown.', 3);

  if Param.Flags and xValidCHS = 0 then begin
    Warning('Cylinder/Head/Sectors-per-track information unknown.');
    FakeDriveGeometry(Param);
  end
  else if Param.Flags and xMaximumCHS <> 0 then begin
    Warning('Current media Cylinder/Head/Sectors-per-track information unknown.');
    CHStoBlock(Param.Heads, Param.SectorsPerTrack, Param.Cylinders-1, Param.Heads-1, Param.SectorsPerTrack, B);
    B := B + 1;
    if B > Param.Sectors then FakeDriveGeometry(Param);
  end
  else begin
    WriteLn('Cylinders:         ', Param.Cylinders);
    WriteLn('Heads:             ', Param.Heads);
    WriteLn('Sectors per Track: ', Param.SectorsPerTrack);
    WriteLn('Sectors:           ', Param.Sectors:0:0);
    WriteLn('Bytes per Sector:  ', Param.BytesPerSector);
    WriteLn('Capacity:          ', Param.Sectors*Param.BytesPerSector/(1024*1024):0:1, ' MBytes');
    WriteLn;
  end;

  GetMem(TestBuffer, BufLen);

  WriteLn('Average Seek Time: ', GetAverageSeekTime:0:1, ' ms');
  WriteLn('Maximal Seek Time: ', GetMaxSeekTime:0:1, ' ms');
  WriteLn('Track-To-Track Seek Time: ', GetTrackToTrackSeekTime:0:1, ' ms');
  AverageAccess := GetAverageAccessTime;
  WriteLn('Average Access Time: ', AverageAccess:0:1, ' ms');
  WriteLn;

  MaxReadSpeed := GetTrackLinearReadSpeed(1);
  reqRead := xVerify;
  if MaxReadSpeed / GetTrackLinearReadSpeed(1) < 0.98 then
    Warning('The cache transfer limits a data transfer.')
  else
    reqRead := xRead;

  GetLinearReadSpeed(AverageReadSpeed, MinReadSpeed, MaxReadSpeed);
  WriteLn('Average Linear Read Speed: ', AverageReadSpeed/1024:0:1, ' MBytes/s');
  WriteLn('Maximum Linear Read Speed: ', MaxReadSpeed/1024:0:1, ' MBytes/s');
  WriteLn('Minimum Linear Read Speed: ', MinReadSpeed/1024:0:1, ' MBytes/s');
  WriteLn;

  WriteLn('Disk Speed Index: ', AverageReadSpeed/AverageAccess:0:0);
end.
